# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause or the CeCILL-B License
# (see codraft/__init__.py for details)

"""
CodraFT, the Codra Filtering Tool
Simple signal and image processing application based on guiqwt and guidata

Starter
"""

from codraft.app import run

run()
