# -*- coding: utf-8 -*-
#
# Licensed under the terms of the BSD 3-Clause or the CeCILL-B License
# (see codraft/__init__.py for details)

"""
CodraFT
=======

CodraFT is a generic signal and image processing software based on Python scientific
libraries (such as NumPy, SciPy or scikit-image) and Qt graphical user interfaces
(thanks to guidata and guiqwt libraries).

CodraFT is Copyright © 2022 CEA-CODRA, Pierre Raybaut, and Licensed under the
terms of the BSD 3-Clause License or the CeCILL-B License.
"""

from distutils.core import setup

setup()
